import asyncio
import argparse
import signal
import threading

from http.server import HTTPServer, SimpleHTTPRequestHandler
from .decorator import register_service
from .get_ip import get_ip_address
import socketio

sio = socketio.AsyncClient()


@sio.event
async def connect():
    print("Connected to the host!")


@sio.event
async def disconnect():
    print("Disconnected from the host.")


@sio.event
async def heartbeat():
    print("Received heartbeat from the host.")


async def main(host_ip, host_port):
    print(f"Connecting to host at {host_ip}:{host_port}")
    await sio.connect(f"http://{host_ip}:{host_port}")
    await sio.wait()


def exit_handler(sig, frame):
    global server_should_exit
    server_should_exit.set()


# def start_client(host_ip="0.0.0.0", host_port=4999):
#     @register_service(name="test client", ip=get_ip_address(), port=13373, host_ip=host_ip, host_port=host_port)
#     def server(port=13373):
#         global server_should_exit
#         server_should_exit = threading.Event()

#         httpd = HTTPServer(('', port), SimpleHTTPRequestHandler)
#         print(f"Serving HTTP on 0.0.0.0 port {port} (http://0.0.0.0:{port}/) ...")

#         def serve_forever_nonblocking():
#             while not server_should_exit.is_set():
#                 httpd.handle_request()

#         server_thread = threading.Thread(target=serve_forever_nonblocking)
#         server_thread.start()


#         return server_thread

#     signal.signal(signal.SIGINT, exit_handler)
#     signal.signal(signal.SIGTERM, exit_handler)

#     server_thread = server()

#     try:
#         asyncio.run(main(host_ip, host_port))
#     finally:
#         print("Shutting down test client...")
#         server_should_exit.set()
#         # server_thread.join()

# def start_client(host_ip="0.0.0.0", host_port=4999):
#     @register_service(name="test client", ip=get_ip_address(), port=13373, host_ip=host_ip, host_port=host_port)
#     def server(port=13373):
#         httpd = HTTPServer(('', port), SimpleHTTPRequestHandler)
#         print(f"Serving HTTP on 0.0.0.0 port {port} (http://0.0.0.0:{port}/) ...")
#         httpd.serve_forever()

#     async def cleanup_and_shutdown():
#         try:
#             server()
#         except KeyboardInterrupt:
#             print("Cleaning up and shutting down...")

#     loop = asyncio.get_event_loop()
#     loop.run_until_complete(cleanup_and_shutdown())

# def start_client(host_ip="0.0.0.0", host_port=4999):
#     @register_service(name="test client", ip=get_ip_address(), port=13373, host_ip=host_ip, host_port=host_port)
#     def server(port=13373):
#         httpd = HTTPServer(('', port), SimpleHTTPRequestHandler)
#         print(f"Serving HTTP on 0.0.0.0 port {port} (http://0.0.0.0:{port}/) ...")
#         httpd.serve_forever()

#         signal.signal(signal.SIGINT, exit_handler)
#         signal.signal(signal.SIGTERM, exit_handler)

#     try:
#         server()
#     except (KeyboardInterrupt, asyncio.exceptions.CancelledError):
#         print("Cleaning up and shutting down...")

def start_client(host_ip="0.0.0.0", host_port=4999):
    @register_service(name="test client", ip=get_ip_address(), port=13373, host_ip=host_ip, host_port=host_port)
    def server(port=13373):
        httpd = HTTPServer(('', port), SimpleHTTPRequestHandler)
        print(f"Serving HTTP on 0.0.0.0 port {port} (http://0.0.0.0:{port}/) ...")

        def serve_forever_nonblocking():
            while not server_should_exit.is_set():
                httpd.handle_request()

            httpd.shutdown()
            httpd.server_close()

        server_thread = threading.Thread(target=serve_forever_nonblocking)
        server_thread.start()

        return server_thread

    server_should_exit = threading.Event()
    signal.signal(signal.SIGINT, exit_handler)
    signal.signal(signal.SIGTERM, exit_handler)


    try:
        server_thread = server()
        asyncio.run(main(host_ip, host_port))
    except (KeyboardInterrupt, asyncio.exceptions.CancelledError):
        print("Cleaning up and shutting down...")
    finally:
        print("Shutting down test client...")
        server_should_exit.set()
        # server_thread.join()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Start announce_server client.")
    parser.add_argument(
        "--host-ip",
        type=str,
        default="0.0.0.0",
        help="Host IP address (default: 0.0.0.0)",
    )
    parser.add_argument(
        "--host-port", type=int, default=4999, help="Host port number (default: 4999)"
    )
    args = parser.parse_args()

    start_client(args.host_ip, args.host_port)
